#!/usr/bin/env python

import rospy

import math
from tf import transformations
from geometry_msgs.msg import PoseWithCovarianceStamped


class PoseSetter(rospy.SubscribeListener):
    def __init__(self, pose, stamp, publish_time):
        self.pose = pose
        self.stamp = stamp
        self.publish_time = publish_time

    def peer_subscribe(self, topic_name, topic_publish, peer_publish):
        p = PoseWithCovarianceStamped()
        p.header.frame_id = "map"
        p.header.stamp = self.stamp
        p.pose.pose.position.x = self.pose[0]
        p.pose.pose.position.y = self.pose[1]
        (p.pose.pose.orientation.x,
         p.pose.pose.orientation.y,
         p.pose.pose.orientation.z,
         p.pose.pose.orientation.w) = transformations.quaternion_from_euler(0, 0, self.pose[2])
        p.pose.covariance[6*0+0] = 0.5 * 0.5
        p.pose.covariance[6*1+1] = 0.5 * 0.5
        p.pose.covariance[6*3+3] = math.pi/12.0 * math.pi/12.0
        # wait for the desired publish time
        while rospy.get_rostime() < self.publish_time:
            rospy.sleep(0.01)
        peer_publish(p)


if __name__ == '__main__':
    pose = map(float, rospy.myargv()[1:4])
    t_stamp = rospy.Time()
    t_publish = rospy.Time()
    if len(rospy.myargv()) > 4:
        t_stamp = rospy.Time.from_sec(float(rospy.myargv()[4]))
    if len(rospy.myargv()) > 5:
        t_publish = rospy.Time.from_sec(float(rospy.myargv()[5]))
    rospy.init_node('pose_setter', anonymous=True)
    rospy.loginfo("Going to publish pose {} with stamp {} at {}".format(pose, t_stamp.to_sec(), t_publish.to_sec()))
    pub = rospy.Publisher("initialpose", PoseWithCovarianceStamped, PoseSetter(pose, stamp=t_stamp, publish_time=t_publish), queue_size=1)
    rospy.spin()



#!/usr/bin/env python
# Software License Agreement (BSD License)
#
# Copyright (c) 2008, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of the Willow Garage nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

from __future__ import print_function

PKG = 'static_map_server'
NAME = 'consumer'

import sys
import unittest
import time

import rospy
import rostest
from nav_msgs.srv import GetMap


class TestConsumer(unittest.TestCase):
    def __init__(self, *args):
        super(TestConsumer, self).__init__(*args)
        self.success = False

    def callback(self, data):
        print(rospy.get_caller_id(), "I heard %s" % data.data)
        self.success = data.data and data.data.startswith('hello world')
        rospy.signal_shutdown('test done')

    def test_consumer(self):
        rospy.wait_for_service('static_map')
        mapsrv = rospy.ServiceProxy('static_map', GetMap)
        resp = mapsrv()
        self.success = True
        print(resp)
        while not rospy.is_shutdown() and not self.success:  # and time.time() < timeout_t: <== timeout_t doesn't exists??
            time.sleep(0.1)
        self.assert_(self.success)
        rospy.signal_shutdown('test done')

if __name__ == '__main__':
    rostest.rosrun(PKG, NAME, TestConsumer, sys.argv)



#!/usr/bin/env python

'''
This script makes Gazebo less fail by translating gazebo status messages to odometry data.
Since Gazebo also publishes data faster than normal odom data, this script caps the update to 20hz.
Winter Guerra
'''

import rospy
from nav_msgs.msg import Odometry
from geometry_msgs.msg import Pose, Twist, Transform, TransformStamped
from gazebo_msgs.msg import LinkStates
from std_msgs.msg import Header
import numpy as np
import math
import tf2_ros
from ackermann_msgs.msg import AckermannDriveStamped

class OdometryNode:
    # Set publishers
    pub_odom = rospy.Publisher('/odom', Odometry, queue_size=1)

    def __init__(self):
        # init internals
        self.last_received_pose = Pose()
        self.last_received_twist = Twist()
        self.last_recieved_stamp = None
        self.x = -3.5
        self.y = 12.0
        self.yaw = 0.0
        self.current_angular_velocity = 0.0
        self.last_cmd_stamp = None
        self.current_cmd_stamp = None
        self.v = 0.0

        # Set the update rate
        rospy.Timer(rospy.Duration(.05), self.timer_callback)  # 20hz

        self.tf_pub = tf2_ros.TransformBroadcaster()

        rospy.Subscriber('/vesc/high_level/ackermann_cmd_mux/input/nav_0', AckermannDriveStamped, self.cmd_callback)

    def cmd_callback(self, data):
        self.current_cmd_stamp = rospy.Time.now()

        if self.last_cmd_stamp is None:
            self.last_cmd_stamp = self.current_cmd_stamp

        dt = self.current_cmd_stamp.to_sec() - self.last_cmd_stamp.to_sec()
        delta_x = data.drive.speed * math.cos(self.yaw) * dt * 0.5 #adjustable 
        delta_y = data.drive.speed * math.sin(self.yaw) * dt * 0.5 #adjustable

        self.current_angular_velocity = data.drive.speed * math.tan(data.drive.steering_angle) / 1.1 #adjustable

        self.x += delta_x
        self.y += delta_y
        self.yaw += self.current_angular_velocity * dt
        self.v = data.drive.speed * 0.5 #adjustable

        self.last_cmd_stamp = self.current_cmd_stamp

    def timer_callback(self, event):
        if self.last_cmd_stamp is None:
            return

        odom = Odometry()
        odom.header.stamp = rospy.Time.now()
        odom.header.frame_id = 'odom'
        odom.child_frame_id = 'base_link'

        odom.pose.pose.position.x = self.x
        odom.pose.pose.position.y = self.y
        odom.pose.pose.position.z = 0.0
        odom.pose.pose.orientation.x = 0.0
        odom.pose.pose.orientation.y = 0.0
        odom.pose.pose.orientation.z = math.sin(self.yaw / 2.0)
        odom.pose.pose.orientation.w = math.cos(self.yaw / 2.0)

        odom.twist.twist.linear.x = self.v
        odom.twist.twist.linear.y = 0
        odom.twist.twist.angular.z = self.current_angular_velocity

        self.pub_odom.publish(odom)

        t = TransformStamped()
        t.header.stamp = odom.header.stamp
        t.header.frame_id = odom.header.frame_id
        t.child_frame_id = odom.child_frame_id
        t.transform.translation = odom.pose.pose.position
        t.transform.rotation = odom.pose.pose.orientation

        self.tf_pub.sendTransform(t)


# Start the node
if __name__ == '__main__':
    rospy.init_node("gazebo_odometry_node_prj4")
    node = OdometryNode()
    rospy.spin()



#!/usr/bin/env python

import sys
import time
from math import fmod, pi

import unittest
import rospy
import rostest

from tf.msg import tfMessage
from tf.transformations import euler_from_quaternion
from std_srvs.srv import Empty


class TestBasicLocalization(unittest.TestCase):
    def setUp(self):
        self.tf = None
        self.target_x = None
        self.target_y = None
        self.target_a = None

    def tf_cb(self, msg):
        for t in msg.transforms:
            if t.header.frame_id == 'map':
                self.tf = t.transform
                (a_curr, a_diff) = self.compute_angle_diff()
                print 'Curr:\t %16.6f %16.6f %16.6f' % (self.tf.translation.x, self.tf.translation.y, a_curr)
                print 'Target:\t %16.6f %16.6f %16.6f' % (self.target_x, self.target_y, self.target_a)
                print 'Diff:\t %16.6f %16.6f %16.6f' % (
                    abs(self.tf.translation.x - self.target_x), abs(self.tf.translation.y - self.target_y), a_diff)

    def compute_angle_diff(self):
        rot = self.tf.rotation
        a = euler_from_quaternion([rot.x, rot.y, rot.z, rot.w])[2]
        d_a = self.target_a

        return (a, abs(fmod(a - d_a + 5*pi, 2*pi) - pi))

    def test_basic_localization(self):
        global_localization = int(sys.argv[1])
        self.target_x = float(sys.argv[2])
        self.target_y = float(sys.argv[3])
        self.target_a = float(sys.argv[4])
        tolerance_d = float(sys.argv[5])
        tolerance_a = float(sys.argv[6])
        target_time = float(sys.argv[7])

        if global_localization == 1:
            #print 'Waiting for service global_localization'
            rospy.wait_for_service('global_localization')
            global_localization = rospy.ServiceProxy('global_localization', Empty)
            global_localization()

        rospy.init_node('test', anonymous=True)
        while(rospy.rostime.get_time() == 0.0):
            #print 'Waiting for initial time publication'
            time.sleep(0.1)
        start_time = rospy.rostime.get_time()
        # TODO: This should be replace by a pytf listener
        rospy.Subscriber('/tf', tfMessage, self.tf_cb)

        while (rospy.rostime.get_time() - start_time) < target_time:
            #print 'Waiting for end time %.6f (current: %.6f)'%(target_time,(rospy.rostime.get_time() - start_time))
            time.sleep(0.1)

        (a_curr, a_diff) = self.compute_angle_diff()
        print 'Curr:\t %16.6f %16.6f %16.6f' % (self.tf.translation.x, self.tf.translation.y, a_curr)
        print 'Target:\t %16.6f %16.6f %16.6f' % (self.target_x, self.target_y, self.target_a)
        print 'Diff:\t %16.6f %16.6f %16.6f' % (
            abs(self.tf.translation.x - self.target_x), abs(self.tf.translation.y - self.target_y), a_diff)
        self.assertNotEquals(self.tf, None)
        self.assertTrue(abs(self.tf.translation.x - self.target_x) <= tolerance_d)
        self.assertTrue(abs(self.tf.translation.y - self.target_y) <= tolerance_d)
        self.assertTrue(a_diff <= tolerance_a)

if __name__ == '__main__':
    rostest.run('amcl', 'amcl_localization',
                TestBasicLocalization, sys.argv)
